/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|


  Bidouilleur PIC12F683 - exemple de contrle d'un moteur pas--pas.
  lectro-Bidouilleur, Novembre 2017

 */
// INSTRUCTIONS AU COMPILATEUR
#include <xc.h>
#include <pic12f683.h>
#define _XTAL_FREQ                  8000000
#define  test_bit(var, bit)   ((var) & (1 <<(bit)))
#define  set_bit(var, bit)    ((var) |= (1 << (bit)))
#define  clear_bit(var, bit)    ((var) &= ~(1 << (bit)))


// CONFIGURATION BYTES
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTOSCIO oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select bit (MCLR pin function is digital input, MCLR internally tied to VDD)
#pragma config CP = OFF         // Code Protection bit (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)
#pragma config BOREN = OFF      // Brown Out Detect (BOR disabled)
#pragma config IESO = OFF       // Internal External Switchover bit (Internal External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enabled bit (Fail-Safe Clock Monitor is disabled)

// CONSTANTS
#define false						0
#define true						1

// GLOBAL VARIABLES
unsigned char i; //Used as counter
unsigned char j; //Used as counter
unsigned char speed = 25; // dure en millisecondes de chaque pulsation au moteur

//******************************************************************************************************

// Timer routines

void delay_ms(int delay) {
    for (i = 0; i < delay; i++) __delay_ms(1);
}

void delay_100us(int delay) {
    for (i = 0; i < delay; i++) __delay_us(100);
}

void main() {
    //Set CPU to 8MHz Here.
    // Initialization stage
    INTCON = 0b00000000; // disable interrupts
    IOC = 0b00000000; // disable GPIO interrupts on change
    PIE1 = 0b00000000; // disable various interrupts
    OPTION_REG = 0b00001111; // weak pullups global enable,
    WPU = 0b00110111; // individual weak pullups enabled
    OSCCON = 0b01110001; // select internal oscillator and set it to 8Mhz
    T1CON = 0b10110111; // enable timer1 and set prescaler div. by 1
    T2CON = 0b01111010; // disable timer2 and set prescaler div. by 16 and postscaler div. by 16
    CCP1CON = 0b00000101; // Set Capture mode, every rising edge
    ADCON0 = 0b00000000; // disable A/D converter
    ANSEL = 0b00000000; // Disable analog assignment to pins
    CMCON0 = 0b00010111; // Disable comparator, use internal reference, invert output.
    CMCON1 = 0b00000000; // comparator gating timer 1 asynchronously
    VRCON = 0b00100001; // Disable internal comparator Vref, use low range, set Vref to 0.15V
    WDTCON = 0b00010100; // Assign the right prescaler value to the watchdog timer. Keep WTD off for now.
    GPIO = 0b00001100; // Set output pins initially before enabling them.
    TRISIO = 0b00001100; // GPIO first disabled, but assigned as follows:
    //	5-sortie, Phase 4 du moteur
    //	4-sortie, Phase 3 du moteur
    //	3-entre ddie, pas de pullup. Bouton Rotation sens contraire aiguilles.
    //	2-entre, Bouton Rotation sens aiguilles.
    //	1-sortie, Phase 2 du moteur
    //	0-sortie, Phase 1 du moteur

    //Main loop
    while (true) {
        if (!test_bit(GPIO, 2)) {
            set_bit(GPIO, 0);
            delay_ms(speed);
            clear_bit(GPIO, 0);
            set_bit(GPIO, 1);
            delay_ms(speed);
            clear_bit(GPIO, 1);
            set_bit(GPIO, 4);
            delay_ms(speed);
            clear_bit(GPIO, 4);
            set_bit(GPIO, 5);
            delay_ms(speed);
            clear_bit(GPIO, 5);
        } else if (!test_bit(GPIO, 3)) {
            set_bit(GPIO, 5);
            delay_ms(speed);
            clear_bit(GPIO, 5);
            set_bit(GPIO, 4);
            delay_ms(speed);
            clear_bit(GPIO, 4);
            set_bit(GPIO, 1);
            delay_ms(speed);
            clear_bit(GPIO, 1);
            set_bit(GPIO, 0);
            delay_ms(speed);
            clear_bit(GPIO, 0);
        }
    }
}